#include "Platform.h"

#include "McuFont.c"



#include "Colors.h"


//Simple VGA Driver 640x480x8bpp that shows embedded image
struct __using("VGA_SimpleDriver1c.spin2") vga; //RJA:  Visual Studio will show an error here, but it's OK



#define USE_GAMMA 0  //set to 1 to use gamma when creating palette or 0 to not use gamma

#define COLORPAIRS 16 //how many pairs of colors are possible, each pair will use 16 of 256 palette entries for 4-bit antialiased graphics
uint32_t ForeColor[COLORPAIRS], BackColor[COLORPAIRS];
uint32_t clrs;  //which pair using now?

void CLS(uint32_t c)
{
    vga.CLS(c * 16); //first color in 16 palette entries is the background color
}

void Setup()
{
    
    BackColor[0] = NavyBlue;
    ForeColor[0] = Yellow;
    BackColor[1] = White;
    ForeColor[1] = Jade;

    for (int c = 0; c < COLORPAIRS; c++)
    {
        MakePalette16(c);
    }

    vga.start(VGA_BASEPIN);
    clrs = 0;
    CLS(clrs);
    
    printf("VGA Driver started.\n");
    //for (;;) {}
}




//This is what prints the characters on the screen
static void pixel_callback(int16_t x, int16_t y, uint8_t count, uint8_t alpha, void* state)
{
    while (count--)
    {
        /* your code goes here, ex: drawPixel(x, y, alpha, color::black); */
        alpha = alpha & 0xF; //just use lower 4 bits for one of 16 colors
        alpha += clrs * 16; //factor in current color set
        vga.SetPixel(x, y, alpha);// AlphaTable[alpha]);

        //printf("Pixel %d,%d = %d\n", x, y, alpha);/// AlphaTable[alpha]);
        x++;
    }
}






void MakePalette16(uint32_t clr)
{//create 16 palette entries for every possible alpha with current foreground and background colors
    float backr, backg, backb;
    float forer, foreg, foreb;
    uint32_t* p = vga.GetPalette();
    uint32_t BackColor24 = BackColor[clr];// BrightGreen; // NavyBlue;// p[BackColor];
    uint32_t ForeColor24 = ForeColor[clr];// WildStrawberry; // Yellow;// p[ForeColor];

    backr = (BackColor24 >> 16) & 0xFF;
    backg = (BackColor24 >> 8) & 0xFF;
    backb = (BackColor24 >> 0) & 0xFF;
    //printf("BackColor24 = %f, %f, %f\n", backr, backg, backb);
    forer = (ForeColor24 >> 16) & 0xFF;
    foreg = (ForeColor24 >> 8) & 0xFF;
    foreb = (ForeColor24 >> 0) & 0xFF;
    //printf("ForeColor24 = %f, %f, %f\n", forer, foreg, foreb);



    int r, g, b;
    int a;
    for (int a16 = 0; a16 < 16; a16++)
    {
        a = a16 + (a16<<4);

        if (USE_GAMMA)
        {
            r = ((forer * forer * a) + (backr * backr * (255 - a)));
            b = ((foreb * foreb * a) + (backb * backb * (255 - a)));
            g = ((foreg * foreg * a) + (backg * backg * (255 - a)));
            r = __builtin_sqrt((int)(r)) >> 4;
            b = __builtin_sqrt((int)(b)) >> 4;
            g = __builtin_sqrt((int)(g)) >> 4;
        }
        else
        {
            r = (int) ((forer * a / 255.) + (backr * (255. - a) / 255.)) &0xFF;
            g = (int)((foreg * a / 255.) + (backg * (255. - a) / 255.)) &0xFF;
            b = (int)((foreb * a / 255.) + (backb * (255. - a) / 255.)) &0xFF;
        }

        //printf("RGB = %d, %d, %d\n", r, g, b);
        //for (;;) {}

        p[a16+clr*16] = (r << 16) + (g << 8) + b;


    }


}


void main()
{
    Setup();


    int height;
    const struct mf_font_s* font;
    struct mf_scaledfont_s scaledfont;
    options_t options;
    state_t state;





    options.fontname = "DejaVuSans32";
    options.filename = "out.bmp";
    options.text = "The quick brown fox jumps over the lazy dog.";
    options.width = 600;
    options.margin = 5;
    options.scale = 1;
    options.anchor = 0;
    options.alignment = MF_ALIGN_LEFT;
    options.justify = false;


    //Get font list and select first font
    static const struct mf_font_list_s* fontlist = MF_INCLUDED_FONTS;
    if (!fontlist)
    {
        printf("No fonts!\n");
        for (;;) {}
    }
    font = fontlist->font;


    do 
    {//Show all fonts
        int x0 = 0;
        int y0 = 0;
        char s[2];
        s[1] = 0;

        options.fontname = font->short_name;
        printf("Font = %s\n", options.fontname);

        options.text = options.fontname;

        state.options = &options;
        options.width = 600;
        state.width = options.width;
        state.height = height;
        options.anchor = x0;
        state.y = y0;// 400;
        state.font = font;
        mf_wordwrap(font, options.width - 2 * options.margin,
            options.text, line_callback, &state);
        y0 += 32*2;

        for (int c = ' '; c < 192; c++)
        {
            s[0] = c;

            //mf_render_aligned(font, state.options.anchor, state.y,
            //    state.options.alignment, line, count,
            //    character_callback, state);

            //mf_render_aligned(
            //    font,//&mf_rlefont_DejaVuSans32.font,
            //    x0, y0,
            //    MF_ALIGN_LEFT,
            //    s, 1,
            //    font->
            //    line_callback,//font->render_character,//&char_callback_DejaVuSans32, 
            //    NULL);
                /* Allocate and clear the image buffer */
            state.options = &options;
            options.width = 50;
            state.width = options.width;
            state.height = height;
            options.anchor = x0;
            state.y = y0;// 400;
            state.font = font;

            ///* Render the text */
            options.text = s;
            line_callback(s, 1, &state);
            //mf_wordwrap(font, options.width - 2 * options.margin,
            //    options.text, line_callback, &state);

            x0 += 30;
            if (x0 > 590)
            {
                x0 = 0;
                y0 += 34;
            }

        }
        //show line of text on the bottom
        x0 = 0;
        y0 += 34;

            //_waitms(2000); //after a couple seconds
        //options.fontname = "DejaVuSans32";
        options.filename = "out.bmp";
        options.text = "The quick brown fox jumps over the lazy dog.";
        options.width = 600;
        options.margin =  5;
        options.scale = 1;
        options.anchor = 0;
        options.alignment = MF_ALIGN_LEFT;
        options.justify = true;// false;

        //font = mf_find_font(options.fontname);

        //if (!font)
        //{
        //    printf("No such font: %s\n", options.fontname);
        //    //return 2;
        //    for (;;) {}
        //}

        /* Count the number of lines that we need. */
        height = 0;
        mf_wordwrap(font, options.width - 2 * options.margin,
            options.text, count_lines, &height);
        height *= font->height;
        height += 4;

        /* Allocate and clear the image buffer */
        state.options = &options;
        state.width = options.width;
        state.height = height;
        state.y = y0;// 400;
        state.font = font;


        /* Render the text */
        mf_wordwrap(font, options.width - 2 * options.margin,
            options.text, line_callback, &state);

        //next font
        //static const struct mf_font_list_s* fontlist = MF_INCLUDED_FONTS;
        
        fontlist = fontlist->next;
        if (!fontlist)
        {//start over
            printf("Start font list over again...\n");
            fontlist = MF_INCLUDED_FONTS;
        }
        font = fontlist->font;

        sleep(2);
        CLS(0); //clear screen
    } while (1);

    for (;;) {}


    /*  //Eval
    #define PIN_CLK  61
    #define PIN_SS   60
    #define PIN_MOSI 59
    #define PIN_MISO 58
    */


    //Simple P2
    #define PIN_CLK  55
    #define PIN_SS   57
    #define PIN_MOSI 56
    #define PIN_MISO 54

    //Save image (308,278 bytes)
    printf("Saving BMP.\n");
    int r = 0;
    r = mount("/sd", _vfs_open_sdcardx(PIN_CLK, PIN_SS, PIN_MOSI, PIN_MISO));
    if (r != 0) {
        printf("ERROR: got error %d during mount\n", r);
    }
    chdir("/sd");
    FILE* f;
    f=fopen("alpha.bmp", "wb");
    if (!f) {
        printf("File open error.\n");
        for (;;) {}        
    }
    fwrite(vga.GetBitmap(), 1, 308278, f);
    fclose(f);
    printf("Bitmap saved.\n");


    for (;;) {}
    _waitms(100);
    vga.cls(BackColor);

        

}

